import requests
import urllib.parse
import re
import argparse
import sys

class TikTokInfo:
    def __init__(self, url):
        self.url = url
        self._fetch_info()

    def _fetch_info(self):
        # Resolve the final URL
        self.final_url = requests.head(self.url, allow_redirects=True).url
        self.parsed_url = urllib.parse.urljoin(
            self.final_url, urllib.parse.urlparse(self.final_url).path
        )

        # Fetch video information
        info = requests.get(f"https://www.tiktok.com/oembed?url={self.parsed_url}").json()
        self._author = info.get("author_name", "N/A")
        self._title = info.get("title", "N/A")
        self._thumbnail = info.get("thumbnail_url", "N/A")

        # Fetch avatar
        try:
            profile_page = requests.get(
                f"https://www.tiktok.com/@{self._author}",
                headers={"User-Agent": "Mozilla/5.0"},
            ).text
            self._avatar = re.search(
                r'"avatarLarger":"(https:[^"]+)"', profile_page
            ).group(1).replace("\\u002F", "/")
        except Exception:
            self._avatar = "N/A"

    def author(self):
        return self._author

    def title(self):
        return self._title

    def thumbnail(self):
        return self._thumbnail

    def avatar(self):
        return self._avatar

    def endpoint(self):
        return self.parsed_url

def cli():
    parser = argparse.ArgumentParser(
        description="TikTokInfo: Fetch TikTok video details"
    )
    parser.add_argument("-url", type=str, help="TikTok video URL")
    parser.add_argument("-author", action="store_true", help="Display author")
    parser.add_argument("-title", action="store_true", help="Display title")
    parser.add_argument("-thumbnail", action="store_true", help="Display thumbnail")
    parser.add_argument("-avatar", action="store_true", help="Display avatar")
    parser.add_argument("-endpoint", action="store_true", help="Display endpoint URL")
    parser.add_argument("-list", action="store_true", help="Display all information")
    parser.add_argument("-help", action="store_true", help="Show usage guide")

    args = parser.parse_args()

    if args.help:
        print("""
Usage: tiktok_info [OPTIONS]

Options:
  -url <URL>          Specify TikTok video URL
  -author             Show the author of the video
  -title              Show the video title
  -thumbnail          Show the thumbnail URL
  -avatar             Show the avatar URL
  -endpoint           Show the endpoint URL
  -list               Show all video information
  -help               Show this help message

Examples:
  tiktok_info -url https://vt.tiktok.com/ZS6F3MQnL/ -list
  tiktok_info -url https://vm.tiktok.com/ZS6F3MQnL/ -list
        """)
        sys.exit(0)

    if not args.url:
        print("Error: Please provide a TikTok URL using -url")
        sys.exit(1)

    tiktok = TikTokInfo(args.url)

    if args.list:
        print("Author:", tiktok.author())
        print("Title:", tiktok.title())
        print("Thumbnail:", tiktok.thumbnail())
        print("Avatar:", tiktok.avatar())
        print("Endpoint URL:", tiktok.endpoint())
    if args.author:
        print("Author:", tiktok.author())
    if args.title:
        print("Title:", tiktok.title())
    if args.thumbnail:
        print("Thumbnail:", tiktok.thumbnail())
    if args.avatar:
        print("Avatar:", tiktok.avatar())
    if args.endpoint:
        print("Endpoint URL:", tiktok.endpoint())


if __name__ == "__main__":
    cli()
